/* Snownews - A lightweight console RSS newsreader
 * 
 * Copyright 2003 Oliver Feiler <kiza@kcore.de>
 * http://kiza.kcore.de/software/snownews/
 *
 * categories.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "categories.h"

struct categories *first_category = NULL;

/* Compare global category list with string categoryname and return 1 if a
   matching category was found. */
int CategoryListItemExists (char * categoryname) {
	struct categories *category;
	
	for (category = first_category; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			/* Matching category found. */
			return 1;
		}
	}

	return 0;
}


/* The following functions add items to/delete items from the global
   category list. */
void CategoryListAddItem (char * categoryname) {
	struct categories *category;
	struct categories *after_ptr;
	struct categories *before_ptr;
	
	/*
#warning Linked list code seems to work, but may be wrong
	*/
	if (CategoryListItemExists (categoryname)) {
		/* Only increase refcount */
		for (category = first_category; category != NULL; category = category->next_ptr) {
			if (strcasecmp (category->name, categoryname) == 0)
				category->refcount++;
		}
	} else {
		/* Otherwise add item to structure */
		category = malloc (sizeof(struct categories));
		category->name = strdup (categoryname);
		
		if (first_category == NULL) {
			/* First element */
			category->next_ptr = first_category;
			first_category = category;
		} else if (first_category->next_ptr == NULL) {
			/* Second element */
			if (strcasecmp(first_category->name, category->name) >= 0) {
				category->next_ptr = first_category;
				first_category = category;
			} else {
				first_category->next_ptr = category;
				category->next_ptr = NULL;
			}
		} else {
			/* nth elment */
			before_ptr = first_category;
			after_ptr = before_ptr->next_ptr;

			/* Walk through the list. */
			while (1) {
				/* If this is the last element, stop. */
				if (after_ptr == NULL)
					break;

				/* If the next element would be larger than the new one, stop. */
				if (strcasecmp(after_ptr->name, category->name) >= 0 )
					break;

				/* Advance both pointers. */
				after_ptr = after_ptr->next_ptr;
				before_ptr = before_ptr->next_ptr;
			}

			/* Insert it between before_ptr and after_ptr. */
			before_ptr->next_ptr = category;
			category->next_ptr = after_ptr;
		}
		category->refcount = 1;
		category->label = 0;
	}
}

/* Delete item from category list. Decrease refcount for each deletion.
   If refcount hits 0, remove from list. */
void CategoryListDeleteItem (char * categoryname) {
	struct categories *category;
	struct categories *before_ptr = NULL;	/* gcc complains ptr uninit with -O2 */
	
	for (category = first_category; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			/* Check refcount */
			if (category->refcount > 1) {
				/* Still at least one feed defining this category */
				category->refcount--;
				break;
			} else {
				/* Last feed using this category removed, kill off the structure */
				if (category == first_category) {
					first_category = first_category->next_ptr;
					free (category->name);
					free (category);
					break;
				} else {
					before_ptr->next_ptr = category->next_ptr;
					free (category->name);
					free (category);
					break;
				}
			}
		}
		before_ptr = category;
	}
}

/* The following two functions add items to/delete items from a feeds
   category list. */
void FeedCategoryAdd (struct feed * cur_ptr, char * categoryname) {
	struct feedcategories *category;
	struct feedcategories *after_ptr;
	struct feedcategories *before_ptr;
	
	/*
#warning Linked list code seems to work, but may be wrong
	*/
	category = malloc (sizeof(struct feedcategories));
	category->name = strdup (categoryname);
	
	if (cur_ptr->feedcategories == NULL) {
		/* Contains no elements */
		category->next_ptr = cur_ptr->feedcategories;
		cur_ptr->feedcategories = category;
	} else if (cur_ptr->feedcategories->next_ptr == NULL) {
		/* Contains n=1 elements */
		if (strcasecmp(cur_ptr->feedcategories->name, category->name) >= 0) {
			category->next_ptr = cur_ptr->feedcategories;
			cur_ptr->feedcategories = category;
		} else {
			cur_ptr->feedcategories->next_ptr = category;
			category->next_ptr = NULL;
		}
	} else {
		/* Contains n>1 elements */
		
		before_ptr = cur_ptr->feedcategories;
		after_ptr = before_ptr->next_ptr;
		
		/* Walk through the list. */
		while (1) {
			/* If this is the last element, stop. */
			if (after_ptr == NULL)
				break;
			
			/* If the next element would be larger than the new one, stop. */
			if (strcasecmp(after_ptr->name, category->name) >= 0 )
				break;
			
			/* Advance both pointers. */
			after_ptr = after_ptr->next_ptr;
			before_ptr = before_ptr->next_ptr;
		}
	
		/* Insert it between before_ptr and after_ptr. */
		before_ptr->next_ptr = category;
		category->next_ptr = after_ptr;
	}

	/* Add to global category list. */
	CategoryListAddItem (category->name);


}

void FeedCategoryDelete (struct feed * cur_ptr, char * categoryname) {
	struct feedcategories *category;
	struct feedcategories *before_ptr = NULL;
	char *tmpname;
	
	/* categoryname == category->name which will be freed by this function! */
	tmpname = strdup (categoryname);
	
	for (category = cur_ptr->feedcategories; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			if (category == cur_ptr->feedcategories) {
				cur_ptr->feedcategories = cur_ptr->feedcategories->next_ptr;
				free (category->name);
				free (category);
				break;
			} else {
				before_ptr->next_ptr = category->next_ptr;
				free (category->name);
				free (category);
				break;
			}
		}
		before_ptr = category;
	}
	
	/* Decrease refcount in global category list. */
	CategoryListDeleteItem (tmpname);
	
	free (tmpname);
}

/* Compare a feeds category list with string categoryname and return 1 if a
   matching category was found. */
int FeedCategoryExists (struct feed * cur_ptr, char * categoryname) {
	struct feedcategories *category;
	
	for (category = cur_ptr->feedcategories; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			/* Matching category found. */
			return 1;
		}
	}

	return 0;
}

/* Set color label of category */
void CategoryListColors (char * categoryname, char * label) {
	struct categories *category;
	
	for (category = first_category; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			/* Matching category found. */
			if (strlen(label) > 5) {
				category->label = atoi (label+5);
				if ((category->label == 12) ||
					(category->label == 17))
					category->labelbold = 1;
				else
					category->labelbold = 0;
			}
		}
	}
}

/* Return color label number and bold status of given category */
int CategoryGetLabel (char * categoryname, int type) {
	struct categories *category;
	
	for (category = first_category; category != NULL; category = category->next_ptr) {
		if (strcasecmp (category->name, categoryname) == 0) {
			/* Matching category found. */
			if (type == 0)
				return category->label;
			else
				return category->labelbold;
		}
	}
	return -1;
}

